# frozen_string_literal: true
require 'rails_helper'
require 'vcr_config'


module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  include Rack::Test::Methods

  MUTATE_PARAMS = {
    requalification: 'Boolean',
    is_from_form: 'Boolean',
    dalo: 'Boolean',
    entry_date: 'DateTime',
    external_ref: 'String',
    communicate_exact_address: 'Boolean',
    typology: 'String',
    lessor_status: 'String',
    type: 'String',
    age: 'String',
    district: 'String',
    publication_end_date: 'DateTime',
    expiry_days: 'Integer',
    rent_amount: 'Float',
    surface: 'Integer',
    rooms: 'Integer',
    exclusivity_end_date: 'DateTime',
    nb_of_eligible_employees: 'Integer',
    nb_of_requests_from_employees: 'Integer',
    availability_date: 'DateTime',
    description: 'String',
    min_effort_treshold: 'Float',
    priority_public: 'Boolean',
    mobility: 'Boolean',
    bedrooms: 'Integer',
    bathrooms: 'Integer',
    heating_type: 'String',
    hot_water_distribution: 'String',
    floor: 'Integer',
    floors: 'Integer',
    elevators: 'Integer',
    has_gardian: 'Boolean',
    outside: 'String',
    cellars: 'Integer',
    parking_charges: 'Float',
    parking_type: 'String',
    address_complements: 'String',
    pmr: 'Boolean',
    kitchen_type: 'String',
    address: 'String',
    lessor_code: "String",
    country: 'String',
    mutation: 'Boolean',
    threshold_type: 'String',
    threshold_subtype: 'String',
    individual_heating: 'Boolean',
    date_publication_start: 'DateTime',
    date_publication_end: 'DateTime',
    kind: 'String',
    building_type: 'String',
    already_rent: 'Boolean',
    guarantee_deposit: 'Float',
    rental_charges: 'Float',
    offer_status: 'String',
    min_effort_rate: 'Float',
    tense_area: 'Boolean',
    subsidiary_company: 'String',
    rpls_id: 'String',
    operation_name: 'String',
    administrator: 'String',
    rent_with_charges: 'Float',
    administration_area: 'String',
    potential_request_name_town: 'String',
    potential_request_name_county: 'String',
    number_of_companies_30_minutes_around: 'Integer',
    construction_type: 'String',
    address_can_be_shared: 'String',
    qpv: 'Boolean',
    total_floors: 'String',
    address_extended: 'String',
    test_created_at: 'DateTime',
    already_inhabited: 'Boolean',
    new_offer: 'Boolean',
    als_siren: 'Siren',
    postal_code: 'String',
    insee_code: 'String',
    residence_code: 'String',
    residence_title: 'String',
    agency_id: 'String',
    application_following_contact: 'String',
    gardens: 'Integer',
    green_area: 'Boolean',
    condo: 'Boolean',
    furnished_lodging: 'Boolean',
    virtual_visit: 'Boolean',
    duplex: 'Boolean',
    first_quartile: 'String',
    other_quartiles: 'String',
    lessor_convention_reference: 'String',
    disability_accesses: 'Integer',
    terraces_or_balconies: 'Integer',
    mad_als_date: 'DateTime',
    reservation_agreement_ref: 'String',
    dpe_ges: 'String',
    dpe_conso: 'String',
    entities_siret: 'Array',
    commercialized: 'Boolean',
    is_public: 'Boolean',
    agency_name: 'String',
    lessor_contact_name: 'String',
    lessor_email: 'String',
    lessor_phone_number: 'String',
    single_parent: 'Boolean',
    auto_destroy_next_day: 'Boolean',
    accept_low_occupation: 'Boolean',
    delai_reponse: 'Integer',
    reference_mad: 'String',
    lessor_orientations: 'Array',
    delai_reponse: 'Integer',
    effort_rate: 'Float',
    construction_year: 'Date',
    lessor_siren: 'Siren',
    ordered_picture_ids: 'Array',
    picture_urls: [[{"full_size" => 'test.jpg'}], [{"full_size" => 'test2.jpg'}]],
    lessor_id: 'String',
    reserving_siren: ['123456789', '223456789'],
    details: 'String',
  }.freeze

  ACCEPTABLE_PARAMS = %w[
    lessor_code
    entity_id
    channel_keys
    is_from_form
    requalification
    dalo
    als_id
    external_ref
    communicate_exact_address
    typology
    type
    age
    district
    department
    publication_end_date
    expiry_days
    rent_amount
    rent_with_charges
    surface
    rooms
    exclusivity_end_date
    availability_date
    description
    details
    effort_rate
    designation_date_end
    min_effort_treshold
    single_parent
    reserved
    already_rent
    reservation_agreement_ref
    priority_public
    mobility
    bedrooms
    bathrooms
    heating_type
    hot_water_distribution
    floor
    floors
    elevators
    has_elevator
    has_gardian
    outside
    cellars
    has_cellar
    parking_type
    pmr
    application_following_contact
    kitchen_type
    address
    postal_code
    country
    address_complements
    mutation
    individual_heating
    date_publication_start
    date_publication_end
    kind
    building_type
    guarantee_deposit
    rental_charges
    parking_charges
    offer_status
    offer_status_id
    offer_status_updated_at
    lessor_status
    lessor_status_id
    lessor_status_updated_at
    min_revenues
    total_charges
    min_effort_rate
    tense_area
    subsidiary_company
    rpls_id
    operation_name
    administrator
    administration_area
    potential_request_name_town
    potential_request_name_county
    number_of_companies_30_minutes_around
    construction_type
    construction_year
    address_can_be_shared
    qpv
    total_floors
    address_extended
    test_created_at
    already_inhabited
    new_offer
    lessor_name
    lessor_siren
    als_siren
    first_quartile
    postal_code
    longitude
    latitude
    insee_code
    residence_code
    residence_title
    pictures
    pictures_nb
    agency_id
    agency_name
    lessor_contact_name
    lessor_email
    lessor_phone_number
    gardens
    has_garden
    green_area
    terraces_or_balconies
    dpe_ges
    dpe_conso
    condo
    furnished_lodging
    virtual_visit
    duplex
    first_quartile
    other_quartiles
    lessor_convention_reference
    disability_accesses
    has_disability_access
    effort_rate
    lessor_id
    operation_id
    lessor
    entities_siret
    entity_ids
    agency_id
    reference
    release_date
    commercialized
    mad_als_date
    is_mad_updated_at
    auto_destroy_next_day
    accept_low_occupation
    active_reservations_rex_ids
    delai_reponse 
    reference_mad
    lessor_orientations
  ].freeze

  HOUSING_REQUESTS_ACCEPTABLE_PARAMS = %w[
    priority_public
    mutation
    housing_channel_keys
    is_active
    als_id
    request_status
    attributed_housing_district
    attributed_housing_district_postal_code
    old_request_status
    request_status_updated_at
    old_entity_status
    updated_entity_status
    entity_status
    dalo
    asked_mobility
    valid_mobility
    decider
    submitted_to_cal_nb
    sent_to_cal_nb
    rejected_in_cal_nb
    rejected_after_cal_nb
    accepted_in_cal_nb
    mobility
    seeked_types
    entity_help_required
    contract_type
    registration_number
    applicant_ids
    entity_validated
    priority_points_indexes
    nud
    nb_of_requests_made
    reference_tax_revenues
    seeked_districts
    category_type
    single_parent
    id_counter
    id_unique
    id_unique_associate
    internal_ref
    forcing_duplicate
    electronical_renewal
    ec_law_ressources
    is_long_delay_anormal
    date_calculator_indicator
    with_car_park
    widen_request
    housing_type_list
    expulsion_judgement_date
    other_reason
    work_contract_type
    seeked_district_postal_codes
    seeked_departments
    seeked_localisation_list
    dependents
    monthly_revenues
    typology_range
    entity_validated
    coapplicants
    lessor_name
    lessor_siren
    social_housing_entry_date
    social_housing_name
    test_created_at
    sne_number
    employee_id
    employee_start_date
    entity_access_level
    employee_unit
    tab_state
    motivations
    seeking_first_floor
    seeking_elevator
    seeked_max_rent
    precisions
    has_multiple_attributions_under_conditions
    seeked_housing_categories
    min_surface
    main_applicant_id
    requester_firstname
    requester_lastname
    requester_fullname
    requester_email
    entity_id
    coapplicant_ids
    coapplicants
    seeked_epci_first_quartils
    seeked_quartil_belongings
    dependents_monthly_incomes
    dependents_monthly_charges
    housing_offer_ids
  ].freeze

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $eq
    $regex
    $all
  ].freeze

  HOUSING_APPLICATIONS_ACCEPTABLE_PARAMS = %w[
    housing_request_id
    housing_offer_id
    epci
    max_date
    applicated_at
    effort_rate
    status_updated_at
  ].freeze

  HOUSING_APPLICATIONS_MUTATE_PARAMS = {
    epci: 'Integer',
    max_date: 'Date',
    status: 'String'
  }.freeze

  describe Api::Dmo::HousingOffersController, type: :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper
    

    check_mutable_params(
      model: Hermes::HousingOffer, 
      mutate_params: MUTATE_PARAMS, 
      factory: :housing_offer,
      min_params: {
        longitude: 0.0,
        latitude: 0.0
      },
      not_checked_keys: [
        :latitude,
        :longitude,
      ],
      other_factory:[
        {factory: :reservation_holder, id: 'test1', params: {siren: ['123456789']} },
        {factory: :reservation_holder, id: 'test2', params: {siren: ['223456789']}},
      ]
    )

    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
    let!(:user) {create(:user)}
    let!(:als_rh) {Hermes::ReservationHolder.create_default}

    describe 'POST /housing_offers' do
      describe 'Create housing_offer with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create',
               data: {
                 attributes: {offer_status: 'EN ATTENTE COMMERCIALISATION' , lessor_status: 'EN ATTENTE PUBLICATION'}
               }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
          perform_enqueued_jobs do
            enqueued_jobs.each do |job|
              if job[:queue] == "exflux5job"
              end
            end
          end
        end
      end

      describe 'Create housing_offer on existing one' do
        let!(:lessor) {create(:lessor, user_id: user.id, title: 'Lessor', siren: "555555555")}
        let!(:older_housing_offer) {create(:housing_offer, user_id: user.id, lessor_siren: "555555555", auto_destroy_next_day: true,
          external_ref: "111", typology: "T1", offer_status: 'RETIRE PAR LE BAILLEUR', lessor_status: 'RETIRE')}
        let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_siren: "555555555", auto_destroy_next_day: true,
                                     external_ref: "111", typology: "T1")}
        let!(:data) do 
          {
            data: {
              attributes: {
                external_ref: "111",
                auto_destroy_next_day: true,
                typology: "T2",
                lessor_siren: "555555555",
                lessor_code: "",
                address: '14 rue edgard quinet',
                postal_code: "30100",
              }
            }
          }
        end
 
        it 'updated existing one' do
   
          housing_offer.set(lessor_code: nil)
       authenticated_header(request, user)
          post 'create',data
          expect(response).to have_http_status(200)
          expect(json['data']['id']).to eq(housing_offer.id.to_s)
        end 
        

        it 'updated existing one with pictures' do
          picture1 = Hermes::Picture.create(housing_offer: housing_offer)
          picture2 = Hermes::Picture.create(housing_offer: housing_offer)
          authenticated_header(request, user)
          expect(housing_offer.reload.ordered_picture_ids).to eq([picture1.id.to_s, picture2.id.to_s])
          post 'create',data
          expect(response).to have_http_status(200)
          expect(json['data']['attributes']["typology"]).to eq("T2")
          expect(housing_offer.reload.ordered_picture_ids).to eq([picture1.id.to_s, picture2.id.to_s])
        end

        it 'update an existing offer without udpating lessor_status, offer_status or reserving_business_name' do 
          params = {
            data: {
              attributes: {
                external_ref: "111",
                lessor_siren: "555555555",
                lessor_code: "",
                typology: 'T2',
                lessor_status: 'lol', 
                offer_status: 'bouh', 
              }
            }
          }
          authenticated_header(request, user)
          post 'create', params
          attributes = params[:data][:attributes]
          expect(response).to have_http_status(200)
          expect(json['data']['id']).to eq(housing_offer.id.to_s)
          expect(json['data']['attributes']["typology"]).to eq(attributes[:typology])
          expect(json['data']['attributes']["lessor_status"]).not_to eq(attributes[:lessor_status])
          expect(json['data']['attributes']["offer_status"]).not_to eq(attributes[:offer_status])
          expect(json['data']['attributes']["reserving_business_name"]).not_to eq(attributes[:reserving_business_name])
        end 

        it 'create new one when create_new_anyway_status' do 
          housing_offer.update(offer_status: 'ARCHIVE')
          authenticated_header(request, user)
          post 'create',data
          expect(response).to have_http_status(201)
          expect(json['data']['id']).not_to eq(housing_offer.id.to_s)
          expect(json['data']['attributes']["typology"]).to eq("T2")
        end 
        it 'not duplicate if lessor_status= "REPRISE BAILLEUR A CONFIRMER"' do
          housing_offer.update(offer_status: 'ARCHIVE', lessor_status: "REPRISE BAILLEUR A CONFIRMER")
          authenticated_header(request, user)
          post 'create',data
          expect(response).to have_http_status(200)
          expect(json['data']['id']).to eq(housing_offer.id.to_s)
        end 
        
      end

      describe 'Create housing_offer autodelete' do
        let!(:lessor) {create(:lessor, user_id: user.id, title: 'Lessor')}

        let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor.id.to_s, auto_destroy_next_day: true,
                                     external_ref: "111", typology: "T1")}
        before :each do
          authenticated_header(request, user)
          post 'create',
               data: {
                 attributes: {
                   lessor_id: lessor.id.to_s,
                   external_ref: "111",
                   auto_destroy_next_day: true,
                   typology: "T2"
                 }
               }
          housing_offer.set(updated_at: Time.now, last_update_by_lessor: Time.now - 37.hours)
          drain_all_jobs

        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
          expect(housing_offer.reload.offer_status).to eq("RETIRE PAR LE BAILLEUR")
          expect(housing_offer.reload.lessor_status).to eq("REPRISE BAILLEUR A CONFIRMER")
          expect(json['data']['id']).to eq(housing_offer.id.to_s)
          expect(json['data']['attributes']["typology"]).to eq("T2")
        end
        
        describe 'POST /housing_offers as an admin' do
          before do
            authenticated_header(request, responsible)
            post :create, data: {
              attributes: {
                offer_status: 'requalification',
                lessor_id: lessor.id.to_s
              }
            }
          end
          it 'can not set reatribution status' do
            expect(response.status).to eq(201)
            expect(json['data']['attributes']['offer_status']).not_to eq('requalification')
            expect(json['data']['attributes']['requalification']).not_to eq(true)
          end
        end
      end

      describe 'with details' do 
        let!(:lessor) {create(:lessor, user_id: user.id, title: 'lessor for details', siren: "123456789")}
        let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_siren: lessor.siren, external_ref: "offer_with_details", typology: "T1", offer_status: 'PUBLIE')}
        before :each do
          authenticated_header(request, user)
        end

        it 'when post data nil' do 
          housing_offer.update(details: 'test')
          post 'create',
               data: {
                 attributes: {
                  lessor_siren: housing_offer.lessor_siren,
                   external_ref: housing_offer.external_ref,
                   details: nil
                 }
               }
          expect(json['data']['id']).to eq(housing_offer.id.to_s)
          expect(housing_offer.reload.details).to eq('test')
        end 

        it 'when post data not blank' do 
          post 'create',
               data: {
                 attributes: {
                  lessor_siren: housing_offer.lessor_siren,
                   external_ref: housing_offer.external_ref,
                   details: 'lol'
                 }
               }
          expect(json['data']['id']).to eq(housing_offer.id.to_s)
          expect(housing_offer.reload.details).to eq('lol')
        end 
      end 

      describe 'PUT /housing_offers/:id as an admin' do
        let!(:lessor_user) {create(:user, :lessor)}
        let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

        before do
          authenticated_header(request, responsible)
          put :update, id: housing_offer.id, data: {
            attributes: {
              offer_status: 'requalification'
            }
          }
        end

        it 'can not set reatribution status' do
          expect(response.status).to eq(200)
          expect(housing_offer.reload.offer_status).not_to eq('requalification')
          expect(housing_offer.reload.requalification).to be(false)
        end
      end

      describe 'PUT /housing_offers/:id republication dépublié as an admin' do
        let!(:lessor_user) {create(:user, :lessor)}
        let!(:housing_offer) {create(:housing_offer,  user_id: user.id, lessor_id: lessor_user.lessor.id.to_s,  lessor_status: "EN DESIGNATION", offer_status: "EN COURS DE DESIGNATION"  , external_ref: "111", typology: "T1", publication_end_date: DateTime.current - 1.days)}

        before do
          housing_offer.set(publication_end_date: Time.zone.now, offer_status: 'COMMERCIALISATION NON PUBLIEE',published: false, updated_at: DateTime.current)
          authenticated_header(request, responsible)
          put :update, id: housing_offer.id, "data":
          {"attributes":
            {"offer_status":"LOGEMENT PUBLIE",
              "lessor_status":"PUBLIE",
              "date_publication_start":
              "Thu Jun 17 00:00:00 CEST 2021",
              "publication_end_date": (DateTime.current + 10.days)}}
        end

        it 'is republished' do
          expect(response.status).to eq(200)
          expect(housing_offer.reload.offer_status).to eq('LOGEMENT PUBLIE')
        end
      end
      describe "pictures" do
        let!(:lessor_user) {create(:user, :lessor)}
        let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

        before do 
          authenticated_header(request, responsible)
        end 

        it 'shows google for default picture' do 
          get :show, id: housing_offer.id
          expect(json['data']['attributes']['main_picture']['thumb240'].match(/googleapis/)).to be_truthy
          expect(json['data']['attributes']['pictures'][0]['thumb240'].match(/googleapis/)).to be_truthy
        end 

        it 'shows picture' do 
          picture = Hermes::Picture.create(housing_offer_id: housing_offer.id, thumb240_id: 'test')
          get :show, id: housing_offer.id
          expect(json['data']['attributes']['main_picture']['thumb240']).to eq('/no_preview_image.png?id=test')
          expect(json['data']['attributes']['pictures'][0]['thumb240']).to eq('/no_preview_image.png?id=test')
        end 
        
        it 'shows pictures_urls in json' do
          put :update, id: housing_offer.id, data: {
            attributes: {
              picture_urls: [{
                thumb240: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
                thumb1000: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
                full_size: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg"
              },
              {
                thumb240: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
                thumb1000: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
                full_size: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg"
              }
            ]
            }
          }
          expect(response.status).to eq(200)
          expect(json['data']['attributes']['main_picture']['thumb240']).to eq("https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg")
          expect(json['data']['attributes']['pictures'][0]['thumb240']).to eq("https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg")
        end

        describe 'PUT with picture_urls' do
          let!(:lessor_user) {create(:user, :lessor)}
          let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

          before do
            authenticated_header(request, responsible)
            put :update, id: housing_offer.id, data: {
              attributes: {
                picture_urls: [{
                  full_size: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
                }
              ]
              }
            }
          end

          it 'shows pictures in json' do
            expect(response.status).to eq(200)
            expect(json['data']['attributes']['main_picture']['thumb240']).to eq("https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg")
            expect(json['data']['attributes']['pictures'][0]['thumb240']).to eq("https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg")
          end
        end
        describe 'PUT with wrong picture urlpicture_urls' do
          let!(:lessor_user) {create(:user, :lessor)}
          let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

          before do
            authenticated_header(request, responsible)
            put :update, id: housing_offer.id, data: {
              attributes: {
                picture_urls: [{
                  foo: "bar",
                }
              ]
              }
            }
          end

          it 'return errors' do
            expect(response.status).to eq(422)
         end
        end
        describe 'PUT with no picture_urls' do
          let!(:lessor_user) {create(:user, :lessor)}
          let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

          before do
            authenticated_header(request, responsible)
            put :update, id: housing_offer.id, data: {
              attributes: {
                picture_urls: nil
              }
            }
          end

          it 'dont return error return errors' do
            expect(response.status).to eq(200)
         end
        end
      end

      describe 'PUT /housing_offers/:id as superadministrateur' do
        let!(:lessor_user) {create(:user, :lessor)}
        let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

        before do
          authenticated_header(request, user)
          put :update, id: housing_offer.id, data: {
            attributes: {
              offer_status: 'requalification'
            }
          }
        end

        it 'can not set reatribution status' do
          expect(response.status).to eq(200)
          expect(housing_offer.reload.offer_status).not_to eq('requalification')
          expect(housing_offer.reload.requalification).to be(false)
        end
      end

      describe 'post request with construction_year is valid' do
        before :each do
          authenticated_header(request, responsible)
          post 'create',
               data: {
                 attributes: {construction_year: '2001'}
               }
        end
        it 'returns status code 201 and the value appears in json' do
          expect(response).to have_http_status(201)
          expect(Date.parse(json['data']['attributes']['construction_year'])).to eq(Date.new(2001, 0o1, 0o1))
        end
      end
      describe 'post request with lessor_id is valid' do
        let!(:lessor) {create(:lessor, user_id: responsible.id, title: 'Lessor')}
        before :each do
          authenticated_header(request, responsible)
          post 'create',
               data: {
                 attributes: {lessor_id: lessor.id.to_s}
               }
        end
        it 'returns status code 201 and the value appears in json' do
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['lessor_id']).to eq(lessor.id.to_s)
        end
      end
    end

    describe 'GET /housing_offers' do
      let!(:housing_offer1) {create(:housing_offer, terraces_or_balconies: 50, description: 'marseille', coordinates: [5.37569223, 43.29360776])}
      let!(:housing_offer3) {create(:housing_offer, terraces_or_balconies: 150, description: 'aix', coordinates: [5.44579067, 43.52887134])}
      let!(:housing_offer2) {create(:housing_offer, terraces_or_balconies: 150, description: 'lyon', coordinates: [4.81934988, 45.78210582])}
      [
        {user: 'responsible', total_objects: 3},
        {user: 'superadministrateur', total_objects: 3},
        {user: 'user', total_objects: 0},
      ].each do |a|
        describe "as #{a[:user]}" do
          before :each do
            authenticated_header(request, eval(a[:user]))
            get 'index'
          end
          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end
          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to eq(a[:total_objects])
            expect(json['meta']['pagination']).to be_present
            expect(json['meta']['pagination']['total_objects']).to eq(a[:total_objects])
          end
        end
      end
      describe 'GET /housing_offers with filters as demanderur with eligibility' do
        let!(:user3) {create(:user, auth_roles: ['demandeur'])}
        let!(:applicant) {create(:applicant, user_id: user3.id.to_s )} #,\"floor_category_6\":{\"$lte\":68817.0}
        let!(:housing_request2) {create(:housing_request, main_applicant_id: applicant.id.to_s, is_active: true)}
        let!(:housing_offer1) {create(:housing_offer, typology: "T6",offer_status: 'PUBLIE')}
        let!(:housing_offer3) {create(:housing_offer, typology: "T2",offer_status: 'PUBLIE')}
        let!(:housing_offer2) {create(:housing_offer, typology: "T4",offer_status: 'PUBLIE')}
        
          describe "as #user with no params" do
            before :each do
              user3.set(main_applicant_id: applicant.id.to_s)
              applicant.set(eligibility_query: "{\"typology\":{\"$in\":[\"T4\",\"T5\",\"T6\",\"T7\",\"T6P\"]}}",updated_at: DateTime.current)
              authenticated_header(request, user3)
              get 'index'
            end
            it 'returns status code 200' do
              expect(response).to have_http_status(200)
            end
            it 'returns a valid json' do
              expect(json).not_to be_empty
              expect(json['data'].length).to eq(2)
              expect(json['meta']['pagination']).to be_present
              expect(json['meta']['pagination']['total_objects']).to eq(2)
            end
          end
          describe "as #user with query params 0"  do
            before :each do
              user3.set(main_applicant_id: applicant.id.to_s)
              applicant.set(eligibility_query: "{\"typology\":{\"$in\":[\"T4\",\"T5\",\"T6\",\"T7\",\"T6P\"]}}",updated_at: DateTime.current)
              authenticated_header(request, user3)
              get 'index', {'typology' => {'$in' => ['T2']}}
            end
            it 'returns status code 200' do
              expect(response).to have_http_status(200)
            end
            it 'returns a valid json' do
              expect(json).not_to be_empty
              expect(json['data'].length).to eq(0)
              expect(json['meta']['pagination']).to be_present
              expect(json['meta']['pagination']['total_objects']).to eq(0)
            end
          end
          describe "as #user with query params" do
            before :each do
              user3.set(main_applicant_id: applicant.id.to_s)
              applicant.set(eligibility_query: "{\"typology\":{\"$in\":[\"T4\",\"T5\",\"T6\",\"T7\",\"T6P\"]}}",updated_at: DateTime.current)
              authenticated_header(request, user3)
              get 'index', {'typology' => {'$in' => ['T2', 'T4']}}
            end
            it 'returns status code 200' do
              expect(response).to have_http_status(200)
            end
            it 'returns a valid json' do
              expect(json).not_to be_empty
              expect(json['data'].length).to eq(1)
              expect(json['meta']['pagination']).to be_present
              expect(json['meta']['pagination']['total_objects']).to eq(1)
            end
          end
        end
  
      describe 'Find by location' do
        let!(:admin) { create(:administrator) }
        before :each do
          Hermes::HousingOffer.remove_indexes
          Hermes::HousingOffer.create_indexes
          authenticated_header(request, admin)
        end

        it 'should return houssing_offers on the selected area' do
          # Coordinates is for Montelimart
          get :index, longitude: 44.55127, latitude: 4.75386, distance: 130
          expect(json['data'].count).to eq(1)
          # Lyon is less than 130 kilometers away
          expect(json['data'].first['id']).to eq(housing_offer3.id.to_s)
        end
      end

      describe 'Find by entity_ids' do
        let!(:admin) { create(:administrator) }
        let!(:lessor_user) {create(:user, :lessor)}
        let!(:housing_offer_with_lessor) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}
        let!(:housing_offer_without_lessor) {create(:housing_offer, user_id: user.id, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

        before :each do
          authenticated_header(request, admin)
        end

        it 'should return houssing_offers on the selected area' do
          post :search, entity_ids: {"$in" => [lessor_user.lessor.id]}
          expect(json['data'].count).to eq(1)
          expect(json['data'].first['id']).to eq(housing_offer_with_lessor.id.to_s)
        end
      end
    end

    describe 'GET /housing_offers/id/all_applications' do
      let!(:housing_offer1) {create(:housing_offer, terraces_or_balconies: 50)}
      let!(:housing_offer2) {create(:housing_offer, terraces_or_balconies: 50)}
      let!(:user) {create(:user)}
      (1..6).each do |i|
        let!("applicant#{i}".to_sym) {create(:applicant, birth_date: Date.today - 50.years)}
        let!("housing_request#{i}".to_sym) {create(:housing_request, is_active: true, motivations: ['acceuillant'], documents_required: 50, main_applicant_id: send("applicant#{i}").id.to_s)}
      end
      let!(:housing_application1) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                          housing_request_id: housing_request1.id.to_s)}
      let!(:housing_application2) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                          housing_request_id: housing_request2.id.to_s)}
      let!(:housing_application3) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                          housing_request_id: housing_request3.id.to_s)}
      let!(:housing_application4) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                          housing_request_id: housing_request4.id.to_s, is_active: false)}
      let!(:housing_application5) {create(:housing_application, housing_offer_id: housing_offer2.id.to_s,
                                          housing_request_id: housing_request5.id.to_s)}
      let!(:housing_application6) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                          housing_request_id: housing_request6.id.to_s)}
    
      it 'return valid json without sort' do
        authenticated_header(request, responsible)
        get 'all_applications', {id: housing_offer1.id}
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(4)
      end
    end

    ACCEPTABLE_PARAMS.each do |acceptable_param|
      type = MUTATE_PARAMS[acceptable_param.to_sym] || MUTATE_PARAMS[acceptable_param.to_s]
      next unless type

      describe 'GET /housing_offers with sort on field ' + acceptable_param + ' works ' do
        let!(:housing_offer3) {create(:housing_offer, acceptable_param => get_typed_value(type, 2))}
        let!(:housing_offer1) {create(:housing_offer, acceptable_param => get_typed_value(type, 0))}
        let!(:housing_offer2) {create(:housing_offer, acceptable_param => get_typed_value(type, 1))}
        before :each do
          authenticated_header(request, responsible)
          get 'index', sort: {acceptable_param => 1}
        end
        it 'returns sorted results' do
          expect(response.status).to eq(200)
          if type == 'DateTime'
            housing_offer_attribute = json['data'].map {|unit| DateTime.parse(unit['attributes'][acceptable_param])}
            expect(housing_offer_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
          elsif type == 'Date'
            housing_offer_attribute = json['data'].map {|unit| Date.parse(unit['attributes'][acceptable_param])}
            expect(housing_offer_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
          else
            housing_offer_attribute = json['data'].map {|unit| unit['attributes'][acceptable_param]}
            expect(housing_offer_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
          end
        end
      end
    end

    describe 'GET /housing_offers with operators' do
      let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50)}
      let!(:housing_offer3) {create(:housing_offer, description: 'Super', terraces_or_balconies: 150)}
      let!(:housing_offer2) {create(:housing_offer, description: 'bel appart', terraces_or_balconies: 150)}
      describe 'with $lt' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', terraces_or_balconies: {'$lt': 100}
        end
        it 'returns results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(1)
        end
      end
      describe 'with $in' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', description: {'$in': %w[acceuillant flamand]}
        end
        it 'returns results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(1)
        end
      end
      describe 'with $ne' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', terraces_or_balconies: {'$ne': 50}
        end
        it 'returns sorted results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(2)
        end
      end
      describe 'with $regex' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', description: {'$regex': /^S/}
        end
        it 'returns sorted results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(1)
        end
      end
    end

    describe 'GET /housing_offers search by postal codes' do
      let!(:offers_1) {create_list :housing_offer, 2, postal_code: '93240'}
      let!(:offers_rand) {create_list :housing_offer, 4, postal_code: '2'.ljust(5, rand(9999).to_s)}
      let!(:offers_2) {create_list :housing_offer, 2, postal_code: '75100'}

      before do
        authenticated_header(request, responsible)
        get 'index', postal_codes: ['75100', '93240']
      end

      it 'succeeds' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq 8
      end

      it 'orders results' do
        expect(json['data'][0]['attributes']['postal_code']).to eq '75100'
        expect(json['data'][2]['attributes']['postal_code']).to eq '93240'
      end
    end

    describe 'POST /housing_offers/search' do
      let!(:housing_offer) {create(:housing_offer, typology: "T1")}
      before :each do
        authenticated_header(request, superadministrateur)
        post 'search', {typology: "T1"}
      end
      
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      
      it 'returns a valid json' do
        expect(json['data'].length).to eq(1)
      end
    end

    describe 'GET /housing_offers/id/eligible_requests' do 
      let!(:entity1) {create(:entity, siren: '123456789', siret: '12345678901234')}
      let!(:entity2) {create(:entity, parent_id: entity1.id,  siret: '22345678901235') }
      let!(:entity3) {create(:entity, siret: '32345678901236') }
      let!(:rh) { create(:reservation_holder, siren: [entity1.siren])}
      let!(:housing_offer1) {create(:housing_offer, typology: 'T1', offer_status: 'PUBLIE') }
      let!(:app1) {create(:applicant, main_employer_siret: entity1.siret)}
      let!(:app2) {create(:applicant, main_employer_siret: entity2.siret)}
      let!(:app3) {create(:applicant, main_employer_siret: entity3.siret)}
      let!(:hr1) { create(:housing_request, is_active: true, main_applicant: app1 ) }
      let!(:hr2) { create(:housing_request, is_active: true, main_applicant: app2 ) }
      let!(:hr3) { create(:housing_request, is_active: true, main_applicant: app3 ) }

      before do :each 
        authenticated_header(request, superadministrateur)
        hr1.set(request_status: 'COMPLETE')
        hr2.set(request_status: 'COMPLETE')
        hr3.set(request_status: 'COMPLETE')
        housing_offer1.set(eligibility_query: {})
      end 

      it 'works' do 
        get 'eligible_requests', id: housing_offer1.id
        expect(json['data'].length).to eq(3)
      end 

      it 'with reserving_siren' do 
        get 'eligible_requests', id: housing_offer1.id, reserving_siren: rh.siren.last
        expect(json['data'].length).to eq(2)
      end 

    end 

    describe 'GET /housing_offers/reference_exist' do
      let!(:housing_offer1) {create(:housing_offer, external_ref: 'test', offer_status: Hermes::HousingOffer.end_of_life_statuses.last)}
      let!(:housing_offer2) {create(:housing_offer, external_ref: 'test', offer_status: 'AUTRE')}
      let!(:housing_offer1) {create(:housing_offer, external_ref: 'bouh',)}

      it 'succeeds' do
        authenticated_header(request, responsible)
        get 'reference_exist', { external_ref: 'test' }
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq 1
      end
    end

    describe 'GET /housing_offers/:id' do
      describe 'verif it work' do
        let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50)}
        before :each do
          authenticated_header(request, responsible)
          get 'show', id: housing_offer1.id
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end

      describe 'verif it work not work with a non authorized user' do
        let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50)}

        before :each do
          authenticated_header(request, user)
          get 'show', id: housing_offer1.id
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end

      context 'with housing applications' do
        let!(:threshold1) do
          create(
            :threshold,
            type: 'PLA LM',
            thresholds_map: {
              'Paris et communes limitrophes': {
                'thresholds_list': [
                  13_050,
                  21_272,
                  27_883,
                  30_521,
                  36_307,
                  40_859
                ],
                'another_person': 4552
              },
              'IDF hors Paris et Communes limitrophes': {
                'thresholds_list': [
                  13_050,
                  21_272,
                  25_569,
                  28_075,
                  33_238,
                  37_401
                ],
                'another_person': 4166
              },
              'Autres régions': {
                'thresholds_list': [
                  11_342,
                  16_525,
                  19_872,
                  22_111,
                  25_870,
                  29_155
                ],
                'another_person': 3252
              }
            }
          )
        end

        let!(:applicant1) do
          create(
            :applicant,
            amount_year_minus_two: 10_346.0,
            monthly_incomes: {
              'SAL': 864.32
            }
          )
        end
        let!(:request1) do
          create(
            :housing_request,
            main_applicant: applicant1,
            is_active: true,
            seeked_departments: %w[12]
          )
        end

        let!(:applicant2) do
          create(
            :applicant,
            amount_year_minus_two: 10_346.0,
            monthly_incomes: {
              'SAL': 864.32
            }
          )
        end
        let!(:request2) do
          create(
            :housing_request,
            main_applicant: applicant2,
            is_active: true,
            seeked_departments: %w[12]
          )
        end

        let!(:applicant3) do
          create(
            :applicant,
            amount_year_minus_two: 10_346.0,
            monthly_incomes: {
              'SAL': 864.32
            }
          )
        end
        let!(:request3) do
          create(
            :housing_request,
            main_applicant: applicant3,
            is_active: true,
            seeked_departments: %w[12]
          )
        end

        let!(:offer1) do
          create(
            :housing_offer,
            department: '12',
            typology: 'T2',
            threshold_type: 'PLA LM',
            threshold_subtype: 'Autres régions',
            rent_with_charges: 324.12
          )
        end
        let!(:offer2) do
          create(
            :housing_offer,
            department: '12',
            typology: 'T2',
            threshold_type: 'PLA LM',
            threshold_subtype: 'Autres régions',
            rent_with_charges: 324.12
          )
        end

        let!(:application1_1) do
          create(
            :housing_application,
            housing_request: request1,
            housing_offer: offer1,
            status: 'TRANSMIS BAILLEUR'
          )
        end
        let!(:application1_2) do
          create(
            :housing_application,
            housing_request: request1,
            housing_offer: offer2,
            status: 'TRANSMIS BAILLEUR'
          )
        end

        let!(:application2_1) do
          create(
            :housing_application,
            housing_request: request2,
            housing_offer: offer1,
            status: 'TRANSMIS BAILLEUR'
          )
        end
        let!(:application2_2) do
          create(
            :housing_application,
            housing_request: request2,
            housing_offer: offer2,
            status: 'TRANSMIS BAILLEUR'
          )
        end

        let!(:application3_1) do
          create(
            :housing_application,
            housing_request: request3,
            housing_offer: offer1,
            status: 'TRANSMIS BAILLEUR'
          )
        end

        let!(:application3_2) do
          create(
            :housing_application,
            housing_request: request3,
            housing_offer: offer1,
            status: 'TRANSMIS BAILLEUR',
            is_active: false,
          )
        end

        describe 'GET /with_validated_applications without from' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications'
          end

          it 'returns error' do
            expect(response).to_not have_http_status(200)
          end
        end

        describe 'GET /with_validated_applications' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications', from: (Time.now - 1.minute).to_s
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to be 2
          end
        end

        describe 'GET /with_validated_applications without pagination' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications', from: (Time.now - 1.minute).to_s, pagination: 'false', per_page: '1'
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to be 2
          end
        end

        describe 'GET /with_validated_applications/offer1.id' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications_for', id: offer1.id.to_s, from: (Time.now - 1.hours).to_s
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to be 3
          end
        end

        describe 'GET /with_validated_applications/offer1.id without pagination' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications_for', id: offer1.id.to_s, from: (Time.now - 1.hours).to_s, pagination: 'false', per_page: '1'
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to be 3
          end
        end

        describe 'GET /with_validated_applications/offer2.id' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications_for', id: offer2.id.to_s, from: (Time.now - 1.hours).to_s
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to be 2
          end
        end
      end
    end

    describe 'GET /housing_offers/:id/entity_statuses' do
      let!(:housing_offer1) {create(:housing_offer)}
      let!(:user1) {create(:user)}
      let!(:user2) {create(:user)}
      let!(:entity1) {create(:entity, siret: '00000000000001', readers:[user1.id.to_s])}
      let!(:entity2) {create(:entity, siret: '00000000000002')}

      let!(:status1) {create(:status,
        object_id: housing_offer1.id.to_s,
        status_label: 'entity_status',
        performer_id: entity1.id.to_s,
        title: 'Status1'
      )}

      let!(:status2) {create(:status,
        object_id: housing_offer1.id.to_s,
        status_label: 'entity_status',
        performer_id: entity1.id.to_s,
        title: 'Status2'
      )}

      let!(:status3) {create(:status,
        object_id: housing_offer1.id.to_s,
        status_label: 'entity_status',
        performer_id: entity2.id.to_s,
        title: 'Status3'
      )}

      describe 'missing params entity_id' do
        before :each do
          authenticated_header(request, user1)
          get 'entity_statuses', {
            id: housing_offer1.id
          }
        end

        it 'returns status code 406' do
          expect(response).to have_http_status(406)
        end
      end

      describe 'not found' do
        before :each do
          authenticated_header(request, user1)
          get 'entity_statuses', {
            id: 'notfound'
          }
        end

        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end

      describe 'unauthorized' do
        before :each do
          authenticated_header(request, user2)
          get 'entity_statuses', {
            id: housing_offer1.id,
            entity_id: entity1.id
          }
        end

        it 'returns status code 401' do
          expect(response).to have_http_status(401)
        end
      end

      describe 'success' do
        before :each do
          authenticated_header(request, user1)
          get 'entity_statuses', {
            id: housing_offer1.id,
            entity_id: entity1.id
          }
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'valid data' do
          expect(json['data'].length).to eq(2)
        end
      end
    end

    describe 'PATCH /housing_offers/:id' do
      housing_offer_params = {}
      describe 'sanitize_string_as_integer' do 
        let!(:housing_offer) {create(:housing_offer)}

        before :each do
          authenticated_header(request, responsible)
        end

        [1, 'O'].each do |type|
          it "works for #{type.class} #{type}" do 
            patch 'update', {id: housing_offer.id.to_s, data: {attributes: {elevators: type}}}
            expect(housing_offer.reload.elevators).to eq 1
          end 
        end 

        [nil, 0, 'N'].each do |type|
          it "works for #{type.class} #{type}" do 
            patch 'update', {id: housing_offer.id.to_s, data: {attributes: {elevators: type}}}
            expect(housing_offer.reload.elevators).to eq 0
          end 
        end 

      end 
    end

    describe 'PATCH /housing_offers/:id flux 11 & 13' do
      describe 'Update housing_offer offer_status and lessor_status' do
        let!(:applicant) {create(:applicant, user: user)}
        let!(:lessor) {create(:lessor, user_id: user.id, title: 'Lessor', siren: "555555555")}
        let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_siren: "555555555", auto_destroy_next_day: true,
                                     external_ref: "111", typology: "T1")}
        let!(:housing_request) {create(:housing_request, is_active: true, user: user, main_user_id: user.id, main_applicant_id: applicant.id)}
        let!(:application) {create(:housing_application,  housing_offer_id: housing_offer.id, housing_request_id: housing_request.id)}

        context 'flux 11 RENDU AU BAILLEUR' do 
          before :each do
            authenticated_header(request, user)
            patch 'update',
              id: housing_offer.id.to_s,
              data: {
                attributes: {
                  offer_status: 'RENDU AU BAILLEUR',
                  lessor_status: 'RENDU AU BAILLEUR CONFIRME'
                }
              }
            drain_all_jobs
          end
          it 'returns status code 200' do
            expect(response).to have_http_status(200)
            expect(application.reload.status).to eq('ANNULEE BAILLEUR')
            expect(application.reload.is_active).to be(false)
          end
        end
      end
    end


    describe 'patch request with change in ordered_picture_ids is valid' do
      let!(:housing_offer1) {create(:housing_offer, user_id: responsible.id)}
      let!(:picture0) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
      let!(:picture1) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
      let!(:picture2) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
      let!(:picture3) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
      let!(:picture4) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}

      before :each do
        authenticated_header(request, responsible)
        patch 'update',
              id: housing_offer1.id.to_s,
              data: {
                attributes: {
                  ordered_picture_ids: [picture4.id.to_s, picture3.id.to_s, picture2.id.to_s, picture1.id.to_s, picture0.id.to_s]
                }
              }
      end
      it 'returns status code 200 and the value appears in json' do
        expect(response).to have_http_status(200)
        expect(json['data']['attributes']['ordered_picture_ids']).to eq([picture4.id.to_s, picture3.id.to_s, picture2.id.to_s, picture1.id.to_s, picture0.id.to_s])
      end
    end

    describe 'post selected application' do
      let!(:housing_offer1) {create(:housing_offer)}
      let!(:housing_offer2) {create(:housing_offer)}

      let!(:user7) {create(:user)}
      let!(:applicant7) {create(:applicant, user: user7)}
      let!(:housing_request7) {create(:housing_request, is_active: true, user: user7, main_user_id: user7.id, main_applicant_id: applicant7.id.to_s, unique_numbers: ['1'])}
      let!(:housing_applications7) do
        create(:housing_application,
               housing_offer_id: housing_offer1.id.to_s,
               housing_request_id: housing_request7.id.to_s,
               status: 'PLO')
      end
      let!(:user8) {create(:user)}
      let!(:applicant8) {create(:applicant, user: user8)}
      let!(:housing_request8) {create(:housing_request, is_active: true, user: user8, main_user_id: user8.id,  main_applicant_id: applicant8.id.to_s, unique_numbers: ['2'])}
      let!(:housing_applications8) do
        create(:housing_application,
               housing_offer_id: housing_offer1.id.to_s,
               housing_request_id: housing_request8.id.to_s,
               status: 'PLO')
      end

      before :each do
        authenticated_header(request, responsible)
      end
      it 'return valid json' do
        post 'post_selected_applications', id: housing_offer1.id, data: {attributes: {housing_application_ids: [housing_applications7.id, housing_applications8.id]}}
        expect(response).to have_http_status(200)
        expect(json).not_to be_empty
        data = json['data']
        expect(data.length).to eq(2)
        expect(housing_applications7.reload.status).to eq('TRANSMIS BAILLEUR')
        expect(housing_offer1.reload.offer_status).to eq("EN COURS D'ATTRIBUTION")
        expect(housing_offer1.reload.lessor_status).to eq('EN ATTRIBUTION')
      end

      it 'send error when missing params' do
        post 'post_selected_applications', id: housing_offer1.id, data: {attributes: {lol: [housing_applications7.id, housing_applications8.id]}}
        expect(response).to have_http_status(422)
      end

      it 'send errors when not active, found or on housing_offer' do 
        housing_applications7.update(status: 'ANNULEE')
        housing_applications8.update(housing_offer: housing_offer2)
        post 'post_selected_applications', {
          id: housing_offer1.id, 
          data: {
            attributes: {
              housing_application_ids: [
                housing_applications7.id, 
                housing_applications7.id, 
                housing_applications8.id,
                housing_applications8.id,
                'not_an_id'
              ]
            }
          }
        }
        expect(response).to have_http_status(422)
        expect(json[0]).to eq("Candidature (id : #{housing_applications7.id} / als_id : #{housing_applications7.als_id}) inactive, statut : ANNULEE, nud : 1")
        expect(json[1]).to eq("Candidature (id : #{housing_applications8.id} / als_id : #{housing_applications8.als_id}) non associée à cette offre (#{housing_offer1.id}), statut : PLO, nud : 2")
        expect(json[2]).to eq("Candidature (not_an_id) introuvable")
      end 

      it 'update other application' do 
        post 'post_selected_applications', id: housing_offer1.id, data: {attributes: {housing_application_ids: [housing_applications7.id,]}}
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(1)
        expect(housing_applications8.reload.status).to eq('NON RETENUE')
        expect(housing_applications8.reload.is_active).to eq(false)
      end 

    end

    describe 'DELETE /housing_offers/:id' do
      let!(:housing_offer) {create(:housing_offer, user_id: responsible.id.to_s)}
      before :each do
        authenticated_header(request, responsible)

        delete 'destroy',
               id: housing_offer.id.to_s
      end
      it 'returns status code 404' do
        expect(response).to have_http_status(204)
      end
    end

    describe 'Housing channels management' do
      let!(:channel_1) { create :housing_channel, key: 'dalo' }
      let!(:channel_2) { create :housing_channel, key: 'priority_public' }
      let!(:channel_3) { create :housing_channel, key: 'droits_communs' }
      let!(:channel_4) { create :housing_channel, key: 'key1' }
      let!(:channel_5) { create :housing_channel, key: 'key2' }
      let!(:channel_6) { create :housing_channel, key: 'key3' }
      let!(:channel_setting) { create :housing_channel_setting, department: '34', housing_channel_keys: ['dalo', 'priority_public', 'key1', 'key2', 'key3'] }

      let(:roles) { ['superadministrateur', 'demandeur', 'entreprise', 'entreprise administrateur', 'collaborateur offre logement'] }
      let(:reservation) { create :reservation_holder }
      let(:lessor) { create :lessor }
      let(:offer) { create :published_offer, housing_channel_keys: ['dalo'], reservation_holder_id: reservation.id, lessor_id: lessor.id }

      before { authenticated_header(request, user) }

      shared_examples 'can add' do
        it 'requires key parameter' do
          post 'add_housing_channel', { id: offer.id.to_s }
          expect(response).to have_http_status 406
          expect(json['errors'][0]['message']).to eq 'param is missing or the value is empty: key or keys'
        end

        it 'returns not found channel error' do
          post 'add_housing_channel', { id: offer.id.to_s, key: 'unknown' }
          expect(response).to have_http_status 404
          expect(json['errors'][0]['message']).to eq "La filière logement 'unknown' n'existe pas."
        end

        it 'returns already added error' do
          post 'add_housing_channel', { id: offer.id.to_s, key: 'dalo' }
          expect(response).to have_http_status 422
          expect(json['errors'][0]['message']).to eq "La filière logement 'dalo' a déjà été ajoutée."
        end

        it 'returns unavailable for department error' do
          post 'add_housing_channel', { id: offer.id.to_s, key: 'droits_communs' }
          expect(response).to have_http_status 422
          expect(json['errors'][0]['message']).to eq "La filière logement 'droits_communs' n'est pas disponible pour le département '34' recherché."
        end

        it 'succeeds with keys' do
          expect do
            post 'add_housing_channel', { id: offer.id.to_s, keys: ['priority_public'] }
          end.to change { offer.reload.housing_channel_keys }.from(['dalo']).to(['priority_public'])
          expect(response).to have_http_status 200
        end

        it 'succeeds with key' do
          expect do
            post 'add_housing_channel', { id: offer.id.to_s, key: 'priority_public' }
          end.to change { offer.reload.housing_channel_keys }.from(['dalo']).to(['dalo', 'priority_public'])
          expect(response).to have_http_status 200
        end

        it 'succeeds with multiple keys' do
          expect do
            post 'add_housing_channel', { id: offer.id.to_s, keys: ['key1', 'key2', 'key3'] }
          end.to change { offer.reload.housing_channel_keys }.from(['dalo']).to(['key1', 'key2', 'key3'])
          expect(response).to have_http_status 200
        end
      end

      shared_examples 'can remove' do
        it 'requires key parameter' do
          post 'remove_housing_channel', { id: offer.id.to_s }
          expect(response).to have_http_status 406
          expect(json['errors'][0]['message']).to eq 'param is missing or the value is empty: key or keys'
        end

        it 'returns not found channel error' do
          post 'remove_housing_channel', { id: offer.id.to_s, keys: ['unknown'] }
          expect(response).to have_http_status 404
          expect(json['errors'][0]['message']).to eq "La filière logement 'unknown' n'existe pas."
        end

        it 'succeeds' do
          expect do
            post 'remove_housing_channel', { id: offer.id.to_s, keys: ['dalo'] }
          end.to change { offer.reload.housing_channel_keys }.from(['dalo']).to(['dc'])
          expect(response).to have_http_status 200
        end
      end

      shared_examples 'cannot add or remove' do
        it 'returns unauthorized on add' do
          post 'add_housing_channel', { id: offer.id.to_s, keys: ['droits_communs'] }
          expect(response).to have_http_status 401
        end
        it 'returns unauthorized on remove' do
          post 'remove_housing_channel', { id: offer.id.to_s, keys: ['dalo'] }
          expect(response).to have_http_status 401
        end
      end

      context 'as bailleur' do
        let(:user) { create :user, auth_roles: ['bailleur'] }
        it_behaves_like 'cannot add or remove'
        it_behaves_like 'can add' do
          before { lessor.update(user_id: user.id) }
        end
        it_behaves_like 'can remove' do
          before { lessor.update(user_id: user.id) }
        end
      end

      context 'as Administrator' do
        let(:user) { create :user, roles: ['Administrator'] }
        it_behaves_like 'can add'
        it_behaves_like 'can remove'
      end

      context 'as other role' do
        it_behaves_like 'cannot add or remove' do
          let(:user) { create(:user, auth_roles: roles.sample(rand(1..3))) }
        end
      end
    end

    context 'with basic data' do
      let!(:applicant) {create(:applicant, user: user)}
      let!(:housing_request) do
        create(
          :housing_request,
          is_active: true,
          main_applicant: applicant,
          user: user
        )
      end
      let!(:user5) {create(:user, auth_roles: ['collaborateur offre logement'])}
      let!(:lessor) {create(:lessor)}

      let!(:housing_offer1) do
        create(
          :housing_offer,
          user: responsible,
          lessor: lessor
        )
      end
      let!(:housing_offer2) do
        create(
          :housing_offer,
          user: responsible,
          lessor: lessor
        )
      end
      let!(:housing_offer3) do
        create(
          :housing_offer,
          user: responsible,
          lessor: lessor
        )
      end

      describe 'GET /housing_offers' do
        before :each do
          authenticated_header(request, responsible)
          get 'index'
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end
      describe 'GET /housing_offers as collaborateur' do
        before :each do
          authenticated_header(request, user5)
          get 'index'
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end
      describe 'GET /housing_offers/:id' do
        context 'as responsible' do
          before :each do
            authenticated_header(request, responsible)
            get 'show', id: housing_offer1.id
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns valid JSON' do
            subject
            expect(json['data']['id']).to eq(housing_offer1.id.to_s)
            expect(json['data']['attributes']['lessor']['title']).to eq(lessor.title)
            expect(json['data']['attributes'].key?('reference')).to be_truthy
            expect(json['data']['attributes'].key?('release_date')).to be_truthy
          end
        end

        context 'with inaccessible housing_offer' do
          before :each do
            authenticated_header(request, responsible)
            get 'show', id: 'test_unavailable_housing_offer'
          end

          it 'returns status code 404' do
            expect(response).to have_http_status(404)
          end
        end
      end

      describe 'PUT /housing_offers/:id as a lessor' do
        describe 'update my housing_offer' do
          let(:valid_params) do
            {
              id: housing_offer2.id.to_s,
              data: {attributes: {rent_amount: 245.62}}
            }
          end

          before :each do
            authenticated_header(request, responsible)
            put 'update', valid_params
          end

          it 'returns status code 200' do
            expect(response.status).to eq(200)
          end

          it 'updated form' do
            expect(json['data']['id']).to eq(housing_offer2.id.to_s)
            expect(json['data']['attributes']['rent_amount']).to eq(245.62)
          end
        end
      end
    end

    context 'as bailleur' do
      let(:bailleur) {create :user, :lessor}
      let(:housing_offer1) {create(:housing_offer, user_id: bailleur.id)}
      let(:housing_offer2) {create(:housing_offer)}

      describe 'GET /housing_offers/:id/history' do
        before do
          create_list(:lessor_objective_history, 4, housing_offer_id: housing_offer1.id)
          create_list(:lessor_objective_history, 4, housing_offer_id: housing_offer2.id)
        end

        describe 'has access' do
          before do
            authenticated_header(request, bailleur)
            get 'history', id: housing_offer1.id
          end

          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq 4
          end
        end

        describe 'is forbidden to access' do
          before do
            authenticated_header(request, bailleur)
            get 'history', id: housing_offer2.id
          end

          it 'returns error' do
            expect(response.status).to eq(404)
          end
        end
      end

      describe 'POST /housing_offers as a lessor' do
        before do
          authenticated_header(request, bailleur)
          post :create, data: {
            attributes: {
              offer_status: 'requalification',
              lessor_id: bailleur.lessor.id
            }
          }
        end
        it 'can reatribution status' do
          expect(response.status).to eq(201)
          expect(json['data']['attributes']['offer_status']).to eq('requalification')
          expect(json['data']['attributes']['requalification']).to eq(true)
        end
      end

      describe 'PUT /housing_offers/:id as a lessor' do
        before do
          authenticated_header(request, bailleur)
          put :update, id: housing_offer1.id, data: {
            attributes: {
              offer_status: 'requalification',
              lessor_id: bailleur.lessor.id
            }
          }
        end
        it 'can set reatribution status' do
          expect(response.status).to eq(200)
          expect(housing_offer1.reload.offer_status).to eq('requalification')
          expect(housing_offer1.reload.requalification).to be(true)
        end
      end
    end

    describe 'GET /housing_offers/with_recount_eligible_requests_nb' do 
      let!(:offer1) {create(:housing_offer, :with_eligible_requests, requests_count: 5)}
      let!(:e1) {create(:entity, siren: '000000001' )}
      let!(:e2) {create(:entity, siren: '000000002' )}
      let!(:e3) {create(:entity, siren: '000000003' )}
      let!(:rh) {create(:reservation_holder, siren: [e1.siren, e2.siren])}

      before do 
        authenticated_header(request, responsible)
      end 

      it 'with no housing_offer_ids' do 
        get 'with_recount_eligible_requests_nb'
        expect(response).to have_http_status(200)
        expect(json['errors'].length).to eq(1)
      end 

      it 'works for unelibile offer' do 
        offer1.set(eligibility_query: nil)
        get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s]
        expect(response).to have_http_status(200)
        expect(json['data'].first['attributes']['eligible_requests_nb']).to eq(nil)
      end 
      
      it 'with no siren' do 
        get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s]
        expect(response).to have_http_status(200)
        expect(json['data'].first['attributes']['eligible_requests_nb']).to eq(5)
      end 

      it 'with siren' do 
        offer1.eligibles_requests.each_with_index do |hr, i |
          hr.set(entity_id: e1.id) if i == 1
          hr.set(entity_id: e2.id) if i == 2
          hr.set(entity_id: e3.id) if i == 3
        end 

        get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s], reserving_siren: e1.siren
        expect(response).to have_http_status(200)
        expect(json['data'].first['attributes']['eligible_requests_nb']).to eq(2)
      end 

      it 'for user not eligibile' do 
        authenticated_header(request, user)
        get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s]
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(0)
      end 

      describe 'for user not eligible with application' do 
        let!(:ha) { create(:housing_application, housing_offer: offer1, housing_request: Hermes::HousingRequest.first,)}

        it 'works' do 
          authenticated_header(request, user)
          ha.set(user_id: user.id)
          get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s]
          expect(response).to have_http_status(200)
          expect(json['data'].length).to eq(1)
          expect(json['data'].first['id']).to eq(offer1.id.to_s)
        end 

      end

    end 

    describe 'get housing_offers/:id with pictures' do 
      let!(:offer) {create(:housing_offer)}
      let!(:blob) { File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/')).read }
      (1..5).each do |i|
        let!("picture#{i}".to_sym) {create(:picture, housing_offer: offer, thumb240_id: ImageTools.store_preview(blob, 240, 240))}
      end 

      it 'show all pictures' do 
        authenticated_header(request, responsible)
        get 'show', id: offer.id
        attributes = json['data']['attributes']
        expect(attributes['pictures'].length).to eq(5)
        expect(attributes['main_picture']['thumb240']).not_to be_blank
      end 

      it 'show all picture_urls' do 
        offer.set(
          updated_at: DateTime.current,
          ordered_picture_ids: [],
          picture_urls: [
            { 'thumb240': 'http://localhost/test1.jpg'},
            { 'thumb240': 'http://localhost/test2.jpg'},
            { 'thumb240': 'http://localhost/test3.jpg'},
          ]
        ) 
        authenticated_header(request, responsible)
        get 'show', id: offer.id
        attributes = json['data']['attributes']
        expect(attributes['pictures'].length).to eq(3)
        expect(attributes['main_picture']['thumb240']).not_to be_blank
      end 

      it 'default picture' do 
        offer.set(updated_at: DateTime.current, ordered_picture_ids: [])
        authenticated_header(request, responsible)
        get 'show', id: offer.id
        attributes = json['data']['attributes']
        expect(attributes['main_picture']['is_default']).to eq(true)
        expect(attributes['main_picture']['thumb240']).not_to be_blank
      end 
    end 

  end

  
  require_relative '../../swagger_helper'

  describe Api::Dmo::HousingOffersController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_OFFERS = 'API_DMO_HOUSING_OFFERS'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}
    let!(:applicant) {create(:applicant, user: responsible)}
    let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s)}
    let!(:als_rh) {Hermes::ReservationHolder.create_default}
    
    let!(:threshold1) do
      create(
        :threshold,
        type: 'PLA LM',
        thresholds_map: {
          'Paris et communes limitrophes': {
            'thresholds_list': [
              13_050,
              21_272,
              27_883,
              30_521,
              36_307,
              40_859
            ],
            'another_person': 4552
          },
          'IDF hors Paris et Communes limitrophes': {
            'thresholds_list': [
              13_050,
              21_272,
              25_569,
              28_075,
              33_238,
              37_401
            ],
            'another_person': 4166
          },
          'Autres régions': {
            'thresholds_list': [
              11_342,
              16_525,
              19_872,
              22_111,
              25_870,
              29_155
            ],
            'another_person': 3252
          }
        }
      )
    end

    context 'as admin' do
      let!(:admin) {create(:administrator)}

      path '/api/dmo/housing_offers' do
        post 'Create housing_offer' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
 
          json_params[:lessor_id] = {type: :string}
          json_params[:construction_year] = {type: :number}
          parameter(
            name: :params,
            in: :body,
            schema: jsonify_params(json_params),
            description: '<span>' \
              '<div class="opblock opblock-delete" style="padding: 4px; align-items: center; display: flex">' \
              '<span class="opblock-summary-method" style="margin-right: 4px">WARNING</span>' \
              '<span class="no-style">Setting an array of siret within `entities_siret`, will ' \
              ' change its relationships with the corresponding entities.</span></div></span>'
          )

          response '201', 'housing_offer created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes(
                {}
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_offers'
            end
          end
        end
      end

      path '/api/dmo/housing_offers' do
        get 'Retrieves all housing_offers' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: 'ids[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          parameter name: 'postal_codes[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          parameter name: :distance, in: :query, type: :integer, required: false
          parameter name: :unit, in: :query, type: :enum, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = :string
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          # VALID_OPERATORS.each do |valid_operator|
          #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          # end

          security [Bearer: {}]
          paginator

          response '200', 'housing_offers found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let!(:housing_offers) {create_list(:housing_offer, 5)}

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) {housing_offers}
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}' do
        get 'Retrieves a single housing_offer by id' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          response '200', 'housing_offer found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:housing_offer) {create(:housing_offer)}
            let(:id) {housing_offer.id}
            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) {housing_offer}
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}/eligible_requests' do
        get 'Retrieves eligible_requests relative to a given housing_offer' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          parameter name: 'ids[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          security [Bearer: {}]
          HOUSING_REQUESTS_ACCEPTABLE_PARAMS.each do |acceptable_param|
            parameter name: acceptable_param.to_s, in: :query, required: false
          end
          paginator
          response '200', 'eligible_requests found' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:housing_offer) {create(:housing_offer)}
            let(:id) {housing_offer.id}

            include_context 'not containings errors'
            run_test!
          end
        end
      end

      path '/api/dmo/housing_offers/with_validated_applications' do
        let!(:applicants) do
          create_list(
            :applicant,
            5,
            amount_year_minus_two: 10_345.22,
            monthly_incomes: {
              'SAL': 1200.0
            }
          )
        end

        let!(:housing_requests) do
          applicants.map do |applicant|
            create(
              :housing_request,
              main_applicant_id: applicant.id,
              is_active: true,
              seeked_departments: %w[26]
            )
          end
        end
        let!(:housing_offers) do
          create_list(
            :housing_offer,
            5,
            rent_with_charges: 234.28,
            rent_amount: 234.28,
            typology: 'T2',
            department: '26',
            threshold_type: 'PLA LM',
            threshold_subtype: 'Autres régions'
          )
        end
        let!(:housing_applications) do
          count = 0
          housing_offers.map do |offer|
            housing_requests.map do |request|
              create(
                :housing_application,
                housing_request_id: request.id,
                housing_offer_id: offer.id,
                status: 'TRANSMIS BAILLEUR'
              )
              count += 1
            end
          end.flatten
        end



        get 'Retrieves all housing_offers that contains a validated applications from a chosen date date' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: 'from', in: :query, type: :datetime, required: true, description: 'minimal `DateTime` of `status` set'
          parameter name: 'to', in: :query, type: :datetime, required: false, description: 'maximal `DateTime` of `status` set'
          parameter name: 'pagination', in: :query, type: :boolean, required: false, description: 'set to `false`, it deactivates the pagination (returns all objects)'
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = :string
            if MUTATE_PARAMS[acceptable_param]
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          # VALID_OPERATORS.each do |valid_operator|
          #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          # end

          security [Bearer: {}]
          paginator

          response '200', 'housing_offers found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:from) {(Time.now - 3.hours).to_s}
            let(:to) {(Time.now + 0.hours).to_s}
            let(:pagination) {'false'}

            include_context 'with integration test'
            include_context 'valid jsonapi collection without pagination' do
              let(:expected_collection) {housing_offers}
            end
          end
        end

      end

      path '/api/dmo/housing_offers/with_validated_applications/{id}' do
        let!(:applicants) do
          create_list(
            :applicant,
            5,
            amount_year_minus_two: 10_345.22,
            monthly_incomes: {
              'SAL': 1200.0
            }
          )
        end
        let!(:housing_requests) do
          applicants.map do |applicant|
            create(
              :housing_request,
              main_applicant_id: applicant.id,
              is_active: true,
              seeked_departments: %w[26]
            )
          end
        end
        let!(:housing_offer) do
          create(
            :housing_offer,
            rent_with_charges: 234.28,
            rent_amount: 234.28,
            typology: 'T2',
            department: '26',
            threshold_type: 'PLA LM',
            threshold_subtype: 'Autres régions'
          )
        end
        let!(:housing_applications) do
          housing_requests.map do |request|
            create(
              :housing_application,
              housing_request_id: request.id,
              housing_offer_id: housing_offer.id,
              status: 'TRANSMIS BAILLEUR'
            )
          end.flatten
        end

        # TODO : wierd bug
        # get 'For a housing_offer, list all housing_application that contains "TRANSMIS BAILLEUR" as their status - from a chosen date date' do
        #   tags TAGS_API_DMO_HOUSING_OFFERS
        #   produces 'application/json'
        #   parameter name: 'id', in: :path, type: :string, required: true, description: 'targeted housing_offer id'
        #   parameter name: 'from', in: :query, type: :datetime, required: true, description: 'minimal `DateTime` of `status` set'
        #   parameter name: 'to', in: :query, type: :datetime, required: false, description: 'maximal `DateTime` of `status` set'
        #   parameter name: 'pagination', in: :query, type: :boolean, required: false, description: 'set to `false`, it deactivates the pagination (returns all objects)'
        #   HOUSING_APPLICATIONS_ACCEPTABLE_PARAMS.each do |acceptable_param|
        #     type = :string
        #     if HOUSING_APPLICATIONS_MUTATE_PARAMS[acceptable_param]
        #       type = nice_type(MUTATE_PARAMS[acceptable_param])
        #     end
        #     parameter name: acceptable_param.to_s, in: :query, type: type, required: false
        #   end
        #   # VALID_OPERATORS.each do |valid_operator|
        #   #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
        #   # end

        #   security [Bearer: {}]
        #   paginator

        #   response '200', 'housing_offers found' do

        #     let(:Authorization) {"Bearer #{token_for(admin)}"}
        #     let(:id) {housing_offer.id.to_s}
        #     let(:from) {(Time.current - 2.hours).to_s}
        #     let(:to) {(Time.current + 1.hours).to_s}
        #     let(:pagination) {'false'}

        #     include_context 'with integration test'
        #     include_context 'valid jsonapi collection without pagination' do
        #       let(:expected_collection) {housing_applications}
        #     end
        #   end
        # end
      end

      path '/api/dmo/housing_offers/{id}/selected_applications' do
        get 'Retrieves housing_applications relative to a given housing_offer where the status is ...' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          parameter name: 'ids[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          security [Bearer: {}]

          HOUSING_APPLICATIONS_ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = :string
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end

          paginator
          response '200', 'housing_application found' do
            let!(:housing_offer1) {create(:housing_offer)}
            let(:'Content-Type') {'multipart/form-data'}
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:id) {housing_offer1.id}
            let!(:housing_requests) do
              create_list(:housing_request, 5, main_applicant_id: applicant.id.to_s) do |request, i|
                create(:housing_application,
                       housing_offer_id: housing_offer1.id.to_s,
                       housing_request_id: request.id.to_s,
                       status: 'PLO'
                )
                create(:housing_application,
                       housing_offer_id: housing_offer1.id.to_s,
                       housing_request_id: request.id.to_s,
                       status: 'TRANSMIS BAILLEUR'
                )
              end
            end

            run_test!
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do |_example|
              json_data = JSON.parse(response.body)['data']
              expect(json_data.length).to eq(5)
            end
          end
        end
      end

      # Pictures functionalities
      path '/api/dmo/housing_offers/{housing_offer_id}/pictures' do
        post 'create_picture for an housing_offer' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: 'Content-Type', in: :header, type: :string, required: false, description: "must be 'multipart/form-data'"
          consumes 'multipart/form-data'
          # consumes 'application/json'
          parameter name: :housing_offer_id, in: :path, type: :string
          security [Bearer: {}]
          parameter name: :file, in: :formData, type: :file, description: 'picture to upload'
          description 'Send a file with the post action to add a picture to the housing_offer. you can only add 20 pictures, use delete and then post do add others or use put to change one.<br> Edit the housing_offer[ordered_picture_ids] field. the main picture is the first of the array. <br> The main picture is the first of the array'

          response '201', 'picture created' do
            let(:file) do
              Rack::Test::UploadedFile.new([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'), 'mime/type')
            end
            let!(:housing_offer1) {create(:housing_offer)}
            let(:'Content-Type') {'multipart/form-data'}
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:housing_offer_id) {housing_offer1.id}
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json_data = JSON.parse(response.body)['data']
              ho = Hermes::HousingOffer.find_by(id: housing_offer1.id)
              expect(json_data).to be_a Hash
              expect(json_data['id']).to_not be nil
              expect(json_data['type']).to eq 'pictures'
              expect(json_data['attributes']['housing_offer_id']).to eq(housing_offer1.id.to_s)
              expect(json_data['attributes']['thumb240']).to_not be_nil
              expect(
                Hermes::HousingOfferSerializer.new(
                  Hermes::HousingOffer.find_by(id: json_data['attributes']['housing_offer_id']),
                  scope: responsible,
                  root: true
                ).attributes[:pictures_nb]
              ).to eq(1)
              expect(ho.ordered_picture_ids).to eq([json_data['id'].to_s])
            end
          end
        end
      end
      path '/api/dmo/housing_offers/{housing_offer_id}/pictures/{picture_id}' do
        put 'edit_picture for an housing_offer' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          parameter name: :housing_offer_id, in: :path, type: :string
          parameter name: :picture_id, in: :path, type: :string
          produces 'application/json'
          parameter name: 'Content-Type', in: :header, type: :string, required: false, description: "must be 'multipart/form-data'"
          consumes 'multipart/form-data'
          parameter name: :file, in: :formData, type: :file, description: 'picture to upload'
          security [Bearer: {}]
          description 'Send a file with the update action to edit one of the picture. if you want to reorder picture, edit the housing_offer[ordered_picture_ids] field. the main picture is the first of the array'

          response '200', 'picture edited' do
            let(:file) do
              Rack::Test::UploadedFile.new([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'), 'mime/type')
            end
            let!(:housing_offer1) {create(:housing_offer)}
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:housing_offer_id) {housing_offer1.id}
            let(:picture1) {create(:picture, housing_offer_id: housing_offer1.id, user_id: responsible.id.to_s)}
            let(:picture_id) {picture1.id.to_s}
            let(:old_assets_ids) {[picture1.thumb240_id, picture1.thumb1000_id, picture1.full_size_id]}
            let(:'Content-Type') {'multipart/form-data'}
            # include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do |_example|
              json_data = JSON.parse(response.body)['data']
              expect(json_data).to be_a Hash
              expect(json_data['id']).to_not be nil
              edited_pic = Hermes::Picture.find_by(id: picture1.id)
              expect(json_data['type']).to eq 'pictures'
              expect(edited_pic.thumb240_id).not_to eq(old_assets_ids[0])
              expect(edited_pic.thumb1000_id).not_to eq(old_assets_ids[1])
              expect(edited_pic.full_size_id).not_to eq(old_assets_ids[2])
            end
          end
        end
      end
      path '/api/dmo/housing_offers/{housing_offer_id}/pictures/{picture_id}' do
        delete 'delete_picture' do
          let!(:housing_offer1) {create(:housing_offer)}
          let(:Authorization) {"Bearer #{token_for(responsible)}"}
          let(:housing_offer_id) {housing_offer1.id}
          let(:picture1) {create(:picture, user_id: responsible.id, housing_offer_id: housing_offer1.id)}

          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :housing_offer_id, in: :path, type: :string
          parameter name: :picture_id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          json_params[:description] = {type: :string}
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '204', 'picture deleted' do
            let(:picture_id) {picture1.id.to_s}
            let(:params) do
              jsonapi_attributes(
                {
                }
              )
            end
            run_test!
            after do |_example|
              ho = Hermes::HousingOffer.find_by(id: housing_offer1.id)
              expect(ho.ordered_picture_ids).to eq([])
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}' do
        put 'edit housing_offer' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          json_params[:lessor_id] = {type: :string}
          json_params[:construction_year] = {type: :number}
          parameter(
            name: :params,
            in: :body,
            schema: jsonify_params(json_params),
            description: '<span>' \
              '<div class="opblock opblock-delete" style="padding: 4px; align-items: center; display: flex">' \
              '<span class="opblock-summary-method" style="margin-right: 4px">WARNING</span>' \
              '<span class="no-style">Setting an array of siret within `entities_siret`, will ' \
              ' change its relationships with the corresponding entities.</span></div></span>'
          )
          # jsonapi_params( **json_params )

          response '200', 'housing_offers updated' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:housing_offer) {create(:housing_offer)}
            let(:id) {housing_offer.id}
            let(:params) do
              jsonapi_attributes(
                external_ref: 'String'
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_offers'
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}' do
        delete 'Delete a single housing_offer by id' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'housing_offer deleted' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:id) {create(:housing_offer, user_id: responsible.id).id}
            run_test!
          end
        end
      end
    end

    context 'as bailleur' do
      let!(:bailleur) { create :user, roles: ['bailleur'] }
      let!(:housing_channel) { create :housing_channel, key: 'dalo' }
      let!(:lessor) { create :lessor, user_id: bailleur.id }

      path '/api/dmo/housing_offers/{id}/add_housing_channel' do
        post 'Add a new keys to the housing_channel_keys array' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string, required: true
          parameter name: :params, in: :body, schema: {
            type: :object,
            properties: {
              keys: { type: :array, items: { type: :string }, collectionFormat: :multi },
              required: [:keys]
            }
          }
          security [Bearer: {}]
          response '200', 'housing channel key added' do
            let(:Authorization) { "Bearer #{token_for(bailleur)}" }
            let(:housing_offer) { create(:published_offer, lessor_id: lessor.id) }
            let(:id) { housing_offer.id }
            let(:params) do
              { keys: ['dalo'] }
            end

            include_context 'with integration test'
            include_context 'not containings errors'
            after do |_example|
              JSON.parse(response.body)
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}/remove_housing_channel' do
        post 'Remove key from the housing_channel_keys array' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string, required: true
          parameter name: :params, in: :body, schema: {
            type: :object,
            properties: {
              keys: { type: :array, items: { type: :string }, collectionFormat: :multi },
              required: [:keys]
            }
          }
          security [Bearer: {}]
          response '200', 'housing channel key removed' do
            let(:Authorization) { "Bearer #{token_for(bailleur)}" }
            let(:housing_offer) { create(:published_offer, lessor_id: lessor.id, housing_channel_keys: ['dalo']) }
            let(:id) { housing_offer.id }
            let(:params) do
              { keys: ['dalo'] }
            end

            include_context 'with integration test'
            include_context 'not containings errors'
            after do |_example|
              JSON.parse(response.body)
            end
          end
        end
      end
    end
  end
end
